# -*- coding: utf-8 -*-
class Tax(object):
    
    def __init__(self, document_code, amount):
        
        self.document_code = document_code
        self.amount = amount

class Iva(Tax):

    def __init__(self, document_code, amount, taxable_base):

        super(Iva, self).__init__(document_code, amount)
        self.taxable_base = taxable_base
            
class Tribute(Tax):
    
    def __init__(self, document_code, amount, taxable_base, aliquot):
        
        super(Tribute, self).__init__(document_code, amount)
        self.taxable_base = taxable_base        
        self.aliquot = aliquot


import requests
import xml.etree.ElementTree as ET

URLS = dict(
    homologation='https://cot.test.arba.gov.ar',
    production='https://cot.arba.gov.ar',
)

class XmlResponseHandler(object):
    def handleResponse(self, response):
        xml = ET.fromstring(response.text)
        return self.handleXml(xml)
        
    def handleXml(self, xml):
        res = {}
        for child in xml.iter():
           if xml.tag != child.tag:
               if len(child):
                   res[child.tag] = self.handleXml(child)
               else: 
                   res[child.tag] = child.text        
        return res

class Configuracion(object):
    def __init__(self, user, password, tipo='homologation'):
        self.base_url = URLS[tipo]
        self.user = user
        self.password = password
        self.handler = XmlResponseHandler()

class ClienteHttp(Configuracion):
    def post(self, url, data=None, files=None):
        response = requests.post(f"{self.base_url}{url}", data=data, files=files)
        response.raise_for_status()
        try:
            return self.handler.handleResponse(response)
        except Exception:
            return {'tipoError': 'DATO', 'mensajeError': 'Se produjo un error al intentar leer el XML.'} 



import certifi
from zeep import Client
from requests import Session
from zeep.transports import Transport
from requests.adapters import HTTPAdapter
from urllib3.util.ssl_ import create_urllib3_context

from l10n_ar_api.afip_webservices import config

# Exclude DH ciphers to avoid "dh key too small" errors with AFIP servers
AFIP_CIPHERS = "DEFAULT:!DH"


class AfipHTTPAdapter(HTTPAdapter):
    """ An adapter to block DH ciphers which may not work for *.afip.gov.ar """

    def init_poolmanager(self, *args, **kwargs):
        context = create_urllib3_context(ciphers=AFIP_CIPHERS)
        context.load_verify_locations(certifi.where())
        kwargs["ssl_context"] = context
        return super().init_poolmanager(*args, **kwargs)


class Wsaa(object):

    def __init__(self, homologation=True, url=None):

        if not url:
            self.url = config.authorization_urls.get('homologation') if homologation\
                else config.authorization_urls.get('production')
        else:
            self.url = url

    def login(self, tra):
        """
        :param tra: TRA que se usara para el logeo
        :return: XML con el login autorizado
        """

        try:
            session = Session()
            session.mount('https://', AfipHTTPAdapter())
            transport = Transport(session=session)
            login_fault = Client(self.url, transport=transport).service.loginCms(tra)
        except Exception as e:
            raise Exception("Error al autenticarse\n{}".format(e))

        return login_fault



# -*- coding: utf-8 -*-
from l10n_ar_api.documents import invoice as inv
from datetime import date, datetime
import math

def round_amount(amount, digits):
    """ Función auxiliar para redondear haciendo que todos los casos de .5 se redondeen hacia arriba """
    multiplier = 10 ** digits
    return math.floor(round(amount * multiplier + 0.5, digits)) / multiplier


class ExportationElectronicInvoiceValidator:

    @classmethod
    def validate_invoice(cls, invoice):
        """
        Valida que los campos de la factura cumplan con los requisitos de la AFIP
        :param invoice: Factura a validar, objeto ExportationElectronicInvoice
        """

        cls._validate_document_date(invoice)
        cls._validate_document_code(invoice)
        cls._validate_existent_permission(invoice)
        cls._validate_document_language(invoice)
        cls._validate_exportation_type(invoice)
        cls._validate_amount(invoice)

    @staticmethod
    def _validate_document_date(invoice):

        if invoice.document_date:
            invoice_date = datetime.strptime(invoice.document_date, '%Y%m%d').date()

            if (abs(date.today() - invoice_date).days > 5):
                raise AttributeError("La fecha del documento debe ser\
                    mayor o menor a 5 dias de la fecha de generacion")

    @staticmethod
    def _validate_amount(invoice):
        if invoice.get_total_item_amount() != invoice.get_total_amount():
            raise AttributeError("La Suma de los items de la factura debe ser igual al total de la misma")

    @staticmethod
    def _validate_exportation_type(invoice):
        if invoice.exportation_type not in (1, 2, 4):
            raise AttributeError('Tipo de exportacion invalido')

    @staticmethod
    def _validate_document_language(invoice):
        if invoice.document_language not in (1, 2, 3):
            raise AttributeError("Idioma del comprobante invalido")

    @staticmethod
    def _validate_existent_permission(invoice):
        if invoice.existent_permission not in ('S', 'N', ''):
            raise AttributeError("Permiso de embarque existente invalido")

    @staticmethod
    def _validate_document_code(invoice):
        if invoice.document_code not in (19, 20, 21):
            raise AttributeError("Codigo de tipo de comprobante invalido")


class ExportationElectronicInvoiceItem(object):

    def __init__(self, description):
        self.description = description
        self.quantity = None
        self.measurement_unit = None
        self.unit_price = None
        self.bonification = None

    @property
    def total_price(self):
        assert self.quantity, "No existe cantidad para el item"
        assert self.unit_price, "No existe precio para el item"

        return round_amount(self.quantity * self.unit_price - self.bonification, 2)


class ExportationElectronicInvoice(inv.Invoice):

    def __init__(self, document_code):
        self.exportation_type = None
        self.existent_permission = None
        self.destiny_country = None
        self.destiny_country_cuit = None
        self.tax_id = None
        self.customer_name = None
        self.customer_street = None
        self.document_language = None
        self.incoterms = None
        self.total_amount = None
        self.array_items = []

        self._payment_due_date = None

        # Moneda
        self.mon_id = None
        self.mon_cotiz = None

        # Comprobantes asociados
        self.associated_documents = []

        super(ExportationElectronicInvoice, self).__init__(document_code)

    # Override
    def get_total_amount(self):
        return round_amount(self.total_amount, 2)

    def get_total_item_amount(self):
        total = sum(item.total_price for item in self.array_items)
        return round_amount(total, 2)

    def add_item(self, value):
        self.array_items.append(value)

    def _parse_date(self, value):
        return value.strftime('%Y%m%d')

    @property
    def document_date(self):
        return self._document_date

    @document_date.setter
    def document_date(self, value):
        self._document_date = self._parse_date(value)

    @property
    def payment_due_date(self):
        return self._payment_due_date

    @payment_due_date.setter
    def payment_due_date(self, value):
        self._payment_due_date = self._parse_date(value)



# -*- coding: utf-8 -*-
from zeep import Client, helpers

from .error import AfipError
from l10n_ar_api.afip_webservices import config

class Wgestimbrefiscal(object):
    def __init__(self, access_token, cuit, tipo_agente, rol, homologation=True, url=None):
        if not url:
            self.url = config.service_urls.get('wstesimbrefiscal_homologation') if homologation\
                else config.service_urls.get('wstesimbrefiscal_production')
        else:
            self.url = url

        self.accessToken = access_token
        self.cuit = cuit
        self.tipo_agente = tipo_agente
        self.rol = rol
        self.argWSAutenticacionEmpresa = self._create_auth_request()

        #details
        self.CuitEmpresa = False
        self.CodigoAduana = False
        self.Destinacion = False

    def fill_details(self, cuitempresa, codigoaduana, destination):
        self.CuitEmpresa = cuitempresa
        self.CodigoAduana = codigoaduana
        self.Destinacion = destination

    def check_webservice_status(self):
        """ Consulta el estado de los webservices de AFIP."""

        res = Client(self.url).service.Dummy()

        if res.Errores != None:
            raise AfipError.parse_error(res)
        if res.Resultado.AppServer != 'OK':
            raise Exception('El servidor de aplicaciones no se encuentra disponible. Intente mas tarde.')
        if res.Resultado.DbServer != 'OK':
            raise Exception('El servidor de base de datos no se encuentra disponible. Intente mas tarde.')
        if res.Resultado.AuthServer != 'OK':
            raise Exception('El servidor de auntenticacion no se encuentra disponible. Intente mas tarde.')


    def _create_auth_request(self):
        argWSAutenticacionEmpresa = {
            'Token': self.accessToken.token,
            'Sign': self.accessToken.sign,
            'CuitEmpresaConectada': self.cuit,
            'TipoAgente': self.tipo_agente,
            'Rol': self.rol,
        }
        return argWSAutenticacionEmpresa

    def retrieve_seals(self, avals):
        sample = [{
            'Item': 1,
            'Subitem': 0,
            'PosicionArancelaria': '8517.12.11',
            'Marca': 'Apple',
            'Modelo': 'iPhone 13 Pro',
            'CodigoGS1': '80898',
            'IdTipoProducto': 1,
            'TimbreTIFE': [
                {'IdentificadorUnico': '010928/00/3890', 'NumeroSerie': 'G7TZL374N81B'},
                {'IdentificadorUnico': '010928/00/38902', 'NumeroSerie': 'G7TZL348N81B'}]
            },{
            'Item': 2,
            'Subitem': 0,
            'PosicionArancelaria': '8517.12.11',
            'Marca': 'Apple',
            'Modelo': 'iPhone 13 Pro MAX',
            'CodigoGS1': '80898',
            'IdTipoProducto': 1,
            'TimbreTIFE': [
                {'IdentificadorUnico': '010928/00/3890', 'NumeroSerie': 'G7TZL374N81B'},
                {'IdentificadorUnico': '010928/00/38902', 'NumeroSerie': 'G7TZL348N81B'}]
            }
        ]

        #print (avals)
        ItemSubitemTIFE = list()
        for vals in avals:
            TimbreTIFE = list()
            _TimbreTIFE = Client(self.url).get_type('ns0:TimbreTIFE')
            for val in vals['TimbreTIFE']:
                TimbreTIFE.append(_TimbreTIFE(
                    IdentificadorUnico=val['IdentificadorUnico'],
                    NumeroSerie=val['NumeroSerie']
                ))

            _ListaTimbres = Client(self.url).get_type('ns0:ArrayOfTimbreTIFE')
            ListaTimbres = _ListaTimbres(
                TimbreTIFE=TimbreTIFE
            )
            _ItemSubitemTIFE = Client(self.url).get_type('ns0:ItemSubitemTIFE')
            ItemSubitemTIFE.append(_ItemSubitemTIFE(
                Item=vals['Item'],
                Subitem=vals['Subitem'],
                PosicionArancelaria=vals['PosicionArancelaria'],
                Marca=vals['Marca'],
                Modelo=vals['Modelo'],
                CodigoGS1=vals['CodigoGS1'],
                IdTipoProducto=vals['IdTipoProducto'],
                ListaTimbres=ListaTimbres
            ))
        self.ItemSubitemTIFE = ItemSubitemTIFE

        _ListaItemsSubitems = Client(self.url).get_type('ns0:ArrayOfItemSubitemTIFE')
        ListaItemsSubitems = _ListaItemsSubitems(
            ItemSubitemTIFE=self.ItemSubitemTIFE
        )
        self.ListaItemsSubitems = ListaItemsSubitems


        _argDestinacionTIFE = Client(self.url).get_type('ns0:DestinacionTIFE')
        argDestinacionTIFE = _argDestinacionTIFE(
            CuitEmpresa=self.CuitEmpresa,
            CodigoAduana=self.CodigoAduana,
            Destinacion=self.Destinacion,
            ListaItemsSubitems=self.ListaItemsSubitems
        )

        #----------DEBUG--------------
        from lxml import etree as ET
        client = Client(self.url)
        node = client.create_message(client.service, 'RegistrarTimbradoFiscalElectDest',
            argWSAutenticacionEmpresa=self.argWSAutenticacionEmpresa,
            argDestinacionTIFE=argDestinacionTIFE)
        tree = ET.ElementTree(node)
        tree.write('/tmp/test.xml', pretty_print=True)



        seals_details = Client(self.url).service.RegistrarTimbradoFiscalElectDest(
            argWSAutenticacionEmpresa=self.argWSAutenticacionEmpresa,
            argDestinacionTIFE=argDestinacionTIFE
        )
        if seals_details.Errores != None:
            raise AfipError().parse_error(seals_details)

        res = helpers.serialize_object(seals_details)
        req = helpers.serialize_object(argDestinacionTIFE)
        return res, req
